@@ -4,9 +4,9 @@ use warnings;
 
 use base 'Test2::Compare::Base';
 
-our $VERSION = '0.000117';
+our $VERSION = '0.000120';
 
-use Test2::Util::HashBase qw/inref ending items order for_each_key for_each_val/;
+use Test2::Util::HashBase qw/inref meta ending items order for_each_key for_each_val/;
 
 use Carp qw/croak confess/;
 use Scalar::Util qw/reftype/;
@@ -43,6 +43,8 @@ sub init {
 
 sub name { '<HASH>' }
 
+sub meta_class  { 'Test2::Compare::Meta' }
+
 sub verify {
     my $self = shift;
     my %params = @_;
@@ -55,6 +57,12 @@ sub verify {
     return 1;
 }
 
+sub add_prop {
+    my $self = shift;
+    $self->{+META} = $self->meta_class->new unless defined $self->{+META};
+    $self->{+META}->add_prop(@_);
+}
+
 sub add_field {
     my $self = shift;
     my ($name, $check) = @_;
@@ -92,6 +100,9 @@ sub deltas {
     # Make a copy that we can munge as needed.
     my %fields = %$got;
 
+    my $meta     = $self->{+META};
+    push @deltas => $meta->deltas(%params) if defined $meta;
+
     for my $key (@{$self->{+ORDER}}) {
         my $check  = $convert->($items->{$key});
         my $exists = exists $fields{$key};
